import { world, system } from "@minecraft/server";

const ObeliskTypeIds = ["sndbx_ww:warden_obelisk"];

// Global variable to track the existing instance
let existingObeliskHandler = null;

export class obeliskHandler {
    static subscriptions = [];

    constructor() {
        if (existingObeliskHandler) {
            //world.sendMessage("ObeliskHandler already exists. Cleaning up old instance...");
            existingObeliskHandler.cleanup(); // Remove old subscriptions
        }

        existingObeliskHandler = this; // Store new instance
        this.initialize();
    }

    initialize() {
       // world.sendMessage("Initializing ObeliskHandler...");

        obeliskHandler.subscriptions.push(
            world.afterEvents.playerInteractWithEntity.subscribe((event) => {
                if (ObeliskTypeIds.includes(event.target.typeId)) this.onObeliskInteract(event);
            })
        );

        obeliskHandler.subscriptions.push(
            system.afterEvents.scriptEventReceive.subscribe((event) => {
                if (event.sourceEntity && ObeliskTypeIds.includes(event.sourceEntity.typeId) && event.id === "sndbx_ww:obelisk_signal") {
                   // this.whisperHint(event.sourceEntity);
                }
            })
        );

        obeliskHandler.subscriptions.push(
            world.afterEvents.entitySpawn.subscribe((event) => {
                const entity = event.entity;

                if (entity.typeId !== "sndbx_ww:warden_obelisk") return;

                const { x, y, z } = entity.location;
        
                const newX = Math.floor(x) + 0.5;
                const newZ = Math.floor(z) + 0.5;
                const newY = Math.floor(y);
        
                entity.teleport({ x: newX, y: newY, z: newZ });
                entity.setRotation({x: 0, y:0});

                entity.runCommand("fill ~ ~ ~ ~ ~6 ~ barrier destroy")
            })
        );
        
    }

    cleanup() {
       // world.sendMessage("Cleaning up ObeliskHandler subscriptions...");

        // Unsubscribe from all stored event handlers
        obeliskHandler.subscriptions.forEach((sub) => sub.unsubscribe());
        obeliskHandler.subscriptions = [];
    }

    onObeliskInteract(event) {
        if (!event.itemStack) return;
        const entity = event.target;
        if (entity.typeId === "sndbx_ww:warden_obelisk" && event.itemStack.typeId === "minecraft:echo_shard") {
            let activeCount = entity.getProperty("sndbx_ww:active_count");
            const activeState = entity.getProperty("sndbx_ww:active");
            if (activeCount < 5) {
                activeCount++
                entity.setProperty("sndbx_ww:active_count", activeCount);
                entity.dimension.playSound("block.end_portal_frame.fill", entity.location);
                if (activeCount == 5 && activeState == false){
                    entity.dimension.playSound("respawn_anchor.deplete", entity.location);
                    entity.setProperty("sndbx_ww:active", true);
                    this.startSummoning(entity);
                }
            }
        }
    }

    startSummoning(obelisk) {
        obelisk.playAnimation("startSummon");
        const dimension = obelisk.dimension;
        const location = obelisk.location;

        system.waitTicks(100).then(() => {
            dimension.createExplosion(location, 3, {
                causesFire: false, // Adjust explosion properties as needed
                breaksBlocks: false // Set to false if you don't want blocks to be destroyed
            });
            dimension.spawnParticle("sndb_ww:star_explosion", location)
            obelisk.runCommand("fill ~ ~ ~ ~ ~7 ~ air")
            const boss = dimension.spawnEntity("sndbx_ww:warden_boss", location);
            if (obelisk.isValid()) obelisk.remove();
        });
    }

    whisperHint(obelisk) {
        const radius = 8;
        const obeliskLocation = obelisk.location;

        let obeliskMessage = "sndbx_warden.message.obelisk";
        const nearbyPlayers = world.getDimension(obelisk.dimension.id).getEntities({
            type: "minecraft:player",
            location: obeliskLocation,
            minDistance: 0,
            maxDistance: radius
        });

        const activationState = obelisk.getProperty("sndbx_ww:active_count") || 0;

        if (activationState == 0) {
            nearbyPlayers.forEach((player) => {
                player.runCommand(`tellraw @s {"rawtext":[{"translate":"${obeliskMessage}"}]}`);
            });
        }
    }
}

