import { world, system, EquipmentSlot } from "@minecraft/server";

export function wardenBattleAxe() {
    world.afterEvents.entityHurt.subscribe((event) => {
        const hurtEntity = event.hurtEntity;

        if (!hurtEntity || !hurtEntity.isValid()) {
            //world.sendMessage("Invalid entity in hurt event.");
            return;
        }

        // Check the amount of damage taken
        const damageTaken = event.damage;

        if (damageTaken === undefined) {
            //world.sendMessage("Unable to retrieve damage from the hurt event.");
            return;
        }

        // Get the source of the damage
        const damageSource = event.damageSource;
        const damagingEntity = damageSource?.damagingEntity;

        if (!damagingEntity || damagingEntity.typeId !== "minecraft:player") {
            //world.sendMessage("Damage source is not a player or invalid.");
            return;
        }

        // Check if the damaging entity is using the Warden Battle Axe
        const weapon = damagingEntity.getComponent("equippable")?.getEquipment(EquipmentSlot.Mainhand);

        if (!weapon || weapon.typeId !== "sndbx_ww:warden_battle_axe") {
            //world.sendMessage("The attacker is not using the Warden Battle Axe.");
            return;
        }

        // Apply slowness effect to the hurt entity
        hurtEntity.addEffect("slowness", 20, { amplifier: 15, showParticles: true });
        system.runTimeout(() => {
            if (!hurtEntity.isValid()) return;
            hurtEntity.addEffect("slowness", 100, { amplifier: 1, showParticles: true });
        }, 20);

        // Detect armor by comparing damage taken
        const baseDamage = 13.77; // This should match the weapon's expected damage
        if (damageTaken < baseDamage) {
            //world.sendMessage(`Armor detected! Damage taken: ${damageTaken} (expected: ${baseDamage})`);
            hurtEntity.applyDamage(5); // Additional damage for armored entities
        } else {
            //world.sendMessage(`No armor detected. Damage taken: ${damageTaken}`);
        }
    });
}
