import { world, system, EquipmentSlot } from "@minecraft/server";

export function wardenDagger() {
    world.afterEvents.entityDie.subscribe((event) => {
        const source = event.damageSource;
        if (!source.damagingEntity || source.damagingEntity.typeId !== "minecraft:player") return;

        const equipment = source.damagingEntity.getComponent('equippable')?.getEquipment(EquipmentSlot.Mainhand);
        if (!equipment) return;
        if (equipment.typeId === "sndbx_ww:warden_dagger") {
            const player = source.damagingEntity;

            // Apply effects and fog
            player.addEffect("speed", 120, { amplifier: 1, showParticles: true });
            player.addEffect("instant_health", 1, { amplifier: 0, showParticles: true });
            player.runCommand("fog @s push sndbx_ww:dagger sndbx_ww:dagger");

            // Add a "fog timer" tag for tracking
            player.addTag("sndbx_ww:fog_timer_120");
            player.dimension.playSound("sndbx_ww.dagger_activate", player.location);
        }
    });

    // Run interval to handle the tag clock and remove fog
    system.runInterval(() => {
        const allPlayers = world.getAllPlayers();

        allPlayers.forEach((player) => {
            const fogTag = player.getTags().find(tag => tag.startsWith("sndbx_ww:fog_timer_"));

            if (fogTag) {
                // Extract the timer value from the tag
                const timerValue = parseInt(fogTag.split("_")[3]);

                if (!isNaN(timerValue) && timerValue > 0) {
                    // Decrement the timer
                    player.removeTag(fogTag);
                    player.addTag(`sndbx_ww:fog_timer_${timerValue - 1}`);
                } else if (timerValue === 0) {
                    // Timer expired: remove the fog and clean up
                    player.runCommand("fog @s pop sndbx_ww:dagger");
                    player.removeTag(fogTag);
                }
            }
        });
    }, 1); // Runs every tick
}
