import { world, EquipmentSlot } from "@minecraft/server";

export function wardenHammer() {
    world.afterEvents.entityHitEntity.subscribe((event) => {
        const equipment = event.damagingEntity.getComponent('equippable')?.getEquipment(EquipmentSlot.Mainhand);
        if (!equipment) return;

        if (equipment.typeId === "sndbx_ww:warden_hammer") {
            const player = event.damagingEntity;
            const hitEntity = event.hitEntity;

            

            if (!player || !hitEntity) return;

            // Calculate direction vector for knockback
            const directionX = hitEntity.location.x - player.location.x;
            const directionZ = hitEntity.location.z - player.location.z;

            // Calculate magnitude of the vector
            const magnitude = Math.sqrt(directionX ** 2 + directionZ ** 2);

            if (magnitude > 0) {
                // Normalize the direction vector
                const normalizedX = directionX / magnitude;
                const normalizedZ = directionZ / magnitude;

                // Apply knockback
                const horizontalStrength = 0; // Adjust horizontal strength as needed
                const verticalStrength = 1; // Adjust vertical strength as needed

                hitEntity.applyKnockback(normalizedX, normalizedZ, horizontalStrength, verticalStrength);
                hitEntity.addEffect("slowness", 60, { amplifier: 2, showParticles: true });
                
            }
        }
    });

    world.afterEvents.itemStartUse.subscribe((event) => {
        const itemStack = event.itemStack;
        if (itemStack.typeId !== "sndbx_ww:warden_hammer") return;

        const cooldownComponent = itemStack.getComponent("cooldown");

        // Start the cooldown
        cooldownComponent?.startCooldown(event.source);

        // Get the player and its location
        const player = event.source;

        player.playAnimation("animation.sndbx_ww.player.use_warden_hammer");
        player.dimension.playSound("sndbx_ww.warden_hammer.use", player.location);
        player.runCommand("camerashake add @s 0.04 0.5 rotational");
        


        // Deduct durability
        const durabilityComponent = itemStack.getComponent("durability");
        const durabilityDeduction = 1;
        if (durabilityComponent) {
            if (durabilityComponent.damage + durabilityDeduction >= durabilityComponent.maxDurability) {
                // If durability is maxed out, play break sound and remove the item
                player.runCommand("/playsound random.break @s");
                const equipment = player.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Mainhand);
                equipment.setItem(undefined); // Clear the item from the slot
            } else {
                // Otherwise, increase damage by the deduction amount and update the item
                durabilityComponent.damage += durabilityDeduction;
                const equipment = player.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Mainhand);
                equipment.setItem(itemStack); // Update the item with new durability
            }
        }

        player.dimension.spawnParticle("sndbx_ww:warden_hammer_interact", player.location)
        player.dimension.playSound("sndbx_ww.use_hammer", player.location);

        
        const range = 4;

        const entitiesInRange = player.dimension.getEntities({
            location: player.location,
            maxDistance: range
        });

        const playerX = player.location.x;
        const playerZ = player.location.z;

        entitiesInRange.forEach((entity) => {
            const entityX = entity.location.x;
            const entityZ = entity.location.z;

            const directionX = entityX - playerX;
            const directionZ = entityZ - playerZ;

            const magnitude = Math.sqrt(directionX ** 2 + directionZ ** 2);

            if (magnitude > 0) {
                const normalizedX = directionX / magnitude;
                const normalizedZ = directionZ / magnitude;

                const horizontalStrength = 5;
                const verticalStrength = 1;
                if (entity.typeId === "minecraft:item" || entity.typeId === "minecraft:xp_orb") return;
                entity.applyKnockback(normalizedX, normalizedZ, horizontalStrength, verticalStrength);
                entity.addEffect("slowness", 60, { amplifier: 2, showParticles: true });
                
            }
        });
    });
}
