import { world, EquipmentSlot, system } from "@minecraft/server";

export function wardenShield() {
    system.runInterval(() => {
        const ALL_PLAYERS = world.getAllPlayers();
        ALL_PLAYERS.forEach(player => {
            if (!player.isValid()) return;

            const equipmentMainhand = player.getComponent('equippable')?.getEquipment(EquipmentSlot.Mainhand);
            const equipmentOffhand = player.getComponent('equippable')?.getEquipment(EquipmentSlot.Offhand);

            if (!player.isSneaking || (!equipmentMainhand && !equipmentOffhand)) return;

            // Mainhand Shield Logic
            if (equipmentMainhand && equipmentMainhand.typeId === "sndbx_ww:warden_shield") {
                const cooldownComponent = equipmentMainhand.getComponent("cooldown");
                if (cooldownComponent && cooldownComponent.getCooldownTicksRemaining(player) > 0) {
                    return; // Exit early if the shield is on cooldown
                }
                player.playAnimation("animation.sndbx_ww.player.shield_block_main_hand");
                player.addEffect("resistance", 2, { amplifier: 255, showParticles: false });
                return;
            }

            // Offhand Shield Logic
            if (equipmentOffhand && equipmentOffhand.typeId === "sndbx_ww:warden_shield") {
                const cooldownComponent = equipmentOffhand.getComponent("cooldown");
                if (cooldownComponent && cooldownComponent.getCooldownTicksRemaining(player) > 0) {
                    return; // Exit early if the shield is on cooldown
                }
                player.playAnimation("animation.sndbx_ww.player.shield_block_off_hand");
                player.addEffect("resistance", 2, { amplifier: 255, showParticles: false });
                return;
            }
        });
    }, 0); // Run every tick

    world.afterEvents.entityHitEntity.subscribe((event) => {
        const hitEntity = event.hitEntity;
        const attacker = event.damagingEntity;

        if (!hitEntity.isSneaking || hitEntity.typeId !== "minecraft:player") return;

        // Retrieve shield from both Mainhand and Offhand
        const equipmentMainhand = hitEntity.getComponent('equippable')?.getEquipment(EquipmentSlot.Mainhand);
        const equipmentOffhand = hitEntity.getComponent('equippable')?.getEquipment(EquipmentSlot.Offhand);

        let equipment = null;
        let equipmentSlot = null;

        // Prioritize Mainhand if both slots have shields
        if (equipmentMainhand && equipmentMainhand.typeId === "sndbx_ww:warden_shield") {
            equipment = equipmentMainhand;
            equipmentSlot = hitEntity.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Mainhand);
        } else if (equipmentOffhand && equipmentOffhand.typeId === "sndbx_ww:warden_shield") {
            equipment = equipmentOffhand;
            equipmentSlot = hitEntity.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Offhand);
        }

        if (!equipment || !equipmentSlot) return;

        // Check if the shield is on cooldown
        const cooldownComponent = equipment.getComponent("cooldown");
        if (cooldownComponent && cooldownComponent.getCooldownTicksRemaining(hitEntity) > 0) {
            return; // Exit early if the shield is on cooldown
        }
        hitEntity.dimension.playSound("item.shield.block", hitEntity.location);

        // Apply initial strong slowness effect to freeze the attacker for 1 second
        attacker.addEffect("slowness", 20, { amplifier: 30, showParticles: true });
        attacker.addEffect("weakness", 20, { amplifier: 2, showParticles: true });
        // After 1 second, apply a lighter slowness effect for 5 seconds
        system.runTimeout(() => {
            attacker.addEffect("slowness", 100, { amplifier: 1, showParticles: true });
        }, 20); // 20 ticks = 1 second

        const lowerAmount = 1;
        // Reduce the durability of the ice shield
        const durabilityComponent = equipment.getComponent("durability");
        if (durabilityComponent) {
            if (durabilityComponent.damage + lowerAmount > durabilityComponent.maxDurability) {
                hitEntity.runCommand("playsound random.break @s");
                equipmentSlot.setItem(undefined); // Clear the item from the slot
                return;
            } else {
                durabilityComponent.damage += lowerAmount;
                equipmentSlot.setItem(equipment);
            }
        }

        // Start cooldown for the shield
        if (cooldownComponent) {
            cooldownComponent.startCooldown(hitEntity);
        }

        // Optional: Notify players when the shield effect is triggered
        // world.sendMessage("Ice Shield activated: enemy frozen briefly and slowed.");
    });

    world.afterEvents.projectileHitEntity.subscribe((event) => {
        const hitEntity = event.getEntityHit().entity;
        const attacker = event.source;

        if (!hitEntity.isSneaking || hitEntity.typeId !== "minecraft:player") return;

        // Retrieve shield from both Mainhand and Offhand
        const equipmentMainhand = hitEntity.getComponent('equippable')?.getEquipment(EquipmentSlot.Mainhand);
        const equipmentOffhand = hitEntity.getComponent('equippable')?.getEquipment(EquipmentSlot.Offhand);

        let equipment = null;
        let equipmentSlot = null;

        // Prioritize Mainhand if both slots have shields
        if (equipmentMainhand && equipmentMainhand.typeId === "sndbx_ww:warden_shield") {
            equipment = equipmentMainhand;
            equipmentSlot = hitEntity.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Mainhand);
        } else if (equipmentOffhand && equipmentOffhand.typeId === "sndbx_ww:warden_shield") {
            equipment = equipmentOffhand;
            equipmentSlot = hitEntity.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Offhand);
        }

        if (!equipment || !equipmentSlot) return;

        // Check if the shield is on cooldown
        const cooldownComponent = equipment.getComponent("cooldown");
        if (cooldownComponent && cooldownComponent.getCooldownTicksRemaining(hitEntity) > 0) {
            return; // Exit early if the shield is on cooldown
        }
        hitEntity.dimension.playSound("item.shield.block", hitEntity.location);

        // Apply initial strong slowness effect to freeze the attacker for 1 second
        attacker.addEffect("slowness", 20, { amplifier: 30, showParticles: true });
        attacker.addEffect("weakness", 20, { amplifier: 2, showParticles: true });
        // After 1 second, apply a lighter slowness effect for 5 seconds
        system.runTimeout(() => {
            attacker.addEffect("slowness", 100, { amplifier: 1, showParticles: true });
        }, 20); // 20 ticks = 1 second

        const lowerAmount = 1;
        // Reduce the durability of the ice shield
        const durabilityComponent = equipment.getComponent("durability");
        if (durabilityComponent) {
            if (durabilityComponent.damage + lowerAmount > durabilityComponent.maxDurability) {
                hitEntity.runCommand("playsound random.break @s");
                equipmentSlot.setItem(undefined); // Clear the item from the slot
                return;
            } else {
                durabilityComponent.damage += lowerAmount;
                equipmentSlot.setItem(equipment);
            }
        }

        // Start cooldown for the shield
        if (cooldownComponent) {
            cooldownComponent.startCooldown(hitEntity);
        }

        // Optional: Notify players when the shield effect is triggered
        // world.sendMessage("Ice Shield activated: enemy frozen briefly and slowed.");
    });
}
