import { world, system } from "@minecraft/server";

export function wardenThrowingStarProjectile() {


    world.afterEvents.projectileHitEntity.subscribe((event) => {
        if (!event.projectile) return;
        if (!event.projectile.isValid()) return;

        if (event.projectile.typeId === "sndbx_ww:warden_star_proj") {
            const hitPosition = event.projectile.location; // Get the position of the projectile
            const dimension = event.projectile.dimension; // Get the dimension of the projectile

            event.projectile.remove(); // Remove the projectile

            // Wait for a short duration (e.g., 20 ticks = 1 second)

            // Find all entities within a 6-block radius
            const entities = dimension.getEntities({
                location: hitPosition,
                maxDistance: 15
            });

            dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
            dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
            dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)

            entities.forEach(entity => {
                // Calculate the impulse vector components
                const dx = hitPosition.x - entity.location.x;
                const dz = hitPosition.z - entity.location.z;
                const dy = hitPosition.y - entity.location.y;

                // Calculate the magnitude (distance)
                const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

                // Avoid division by zero
                if (distance === 0) return;

                // Normalize direction components
                const normalizedX = dx / distance;
                const normalizedY = dy / distance;
                const normalizedZ = dz / distance;

                // Define the impulse force (adjust as needed)
                const impulseStrength = 2; // Controls overall force of the impulse
                const impulse = {
                    x: normalizedX * impulseStrength,
                    y: normalizedY * impulseStrength,
                    z: normalizedZ * impulseStrength
                };

                // Apply the impulse to the entity
                if (entity.typeId === "minecraft:player") return;
                entity.applyImpulse(impulse);
            });
            system.runTimeout(() => {
                // Create an explosion at the hit position
                dimension.createExplosion(hitPosition, 4, {
                    causesFire: false, // Adjust explosion properties as needed
                    breaksBlocks: true // Set to false if you don't want blocks to be destroyed
                });
                dimension.spawnParticle("sndb_ww:star_explosion", hitPosition)

            }, 10); // 20 ticks = 1 second delay
        }
    });


    world.afterEvents.projectileHitBlock.subscribe((event) => {
        if (!event.projectile) return;
        if (!event.projectile.isValid()) return;

        if (event.projectile.typeId === "sndbx_ww:warden_star_proj") {
            const hitPosition = event.projectile.location; // Get the position of the projectile
            const dimension = event.projectile.dimension; // Get the dimension of the projectile

            event.projectile.remove(); // Remove the projectile

            dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
            dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
            dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)

            // Find all entities within a 6-block radius
            const entities = dimension.getEntities({
                location: hitPosition,
                maxDistance: 15
            });

            entities.forEach(entity => {
                // Calculate the impulse vector components
                const dx = hitPosition.x - entity.location.x;
                const dz = hitPosition.z - entity.location.z;
                const dy = hitPosition.y - entity.location.y;

                // Calculate the magnitude (distance)
                const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

                // Avoid division by zero
                if (distance === 0) return;

                // Normalize direction components
                const normalizedX = dx / distance;
                const normalizedY = dy / distance;
                const normalizedZ = dz / distance;

                // Define the impulse force (adjust as needed)
                const impulseStrength = 2; // Controls overall force of the impulse
                const impulse = {
                    x: normalizedX * impulseStrength,
                    y: normalizedY * impulseStrength,
                    z: normalizedZ * impulseStrength
                };

                // Apply the impulse to the entity
                if (entity.typeId === "minecraft:player") return;
                entity.applyImpulse(impulse);
            });
            system.runTimeout(() => {
                // Create an explosion at the hit position
                dimension.createExplosion(hitPosition, 4, {
                    causesFire: false, // Adjust explosion properties as needed
                    breaksBlocks: true // Set to false if you don't want blocks to be destroyed
                });
                dimension.spawnParticle("sndb_ww:star_explosion", hitPosition)
            }, 10); // 20 ticks = 1 second delay
        }
    });


    world.afterEvents.entitySpawn.subscribe((event) => {
        const entity = event.entity;

        if (!entity.isValid() || entity.typeId !== "sndbx_ww:warden_star_proj") return;
        const location = event.entity.location;
        //entity.dimension.playSound("sndbx_ww.warden_throwing_staroot", location)

    });

    world.afterEvents.itemStartUse.subscribe((event) => {
        const itemStack = event.itemStack;
        if (itemStack.typeId !== "sndbx_ww:warden_star") return;

        const cooldownComponent = itemStack.getComponent("cooldown");

        // Start the cooldown
        cooldownComponent?.startCooldown(event.source);

        // Get the player and its location
        const player = event.source;
        player.playAnimation("animation.sndbx_ww.player.use_warden_star");
        player.dimension.playSound("sndbx_ww.use_star", player.location);
        //player.dimension.playSound("sndbx_ww.warden_star.use", player.location);

    });
}