import { world, EquipmentSlot, system } from "@minecraft/server";

export function wardenWand() {

  world.afterEvents.projectileHitEntity.subscribe((event) => {
    if (!event.projectile) return;
    if (!event.projectile.isValid()) return;

    if (event.projectile.typeId === "sndbx_ww:warden_wand_proj") {
      const hitPosition = event.projectile.location; // Get the position of the projectile
      const dimension = event.projectile.dimension; // Get the dimension of the projectile

      event.projectile.remove(); // Remove the projectile

      // Wait for a short duration (e.g., 20 ticks = 1 second)

      // Find all entities within a 6-block radius
      const entities = dimension.getEntities({
        location: hitPosition,
        maxDistance: 20
      });

      dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
      dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
      dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)

      entities.forEach(entity => {
        // Calculate the impulse vector components
        const dx = hitPosition.x - entity.location.x;
        const dz = hitPosition.z - entity.location.z;
        const dy = hitPosition.y - entity.location.y;

        // Calculate the magnitude (distance)
        const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

        // Avoid division by zero
        if (distance === 0) return;

        // Normalize direction components
        const normalizedX = dx / distance;
        const normalizedY = dy / distance;
        const normalizedZ = dz / distance;

        // Define the impulse force (adjust as needed)
        const impulseStrength = 8; // Controls overall force of the impulse
        const impulse = {
          x: normalizedX * impulseStrength,
          y: normalizedY * impulseStrength,
          z: normalizedZ * impulseStrength
        };

        // Apply the impulse to the entity
        if (entity.typeId === "minecraft:player") return;
        entity.applyImpulse(impulse);
      });
      system.runTimeout(() => {
        // Create an explosion at the hit position
        dimension.createExplosion(hitPosition, 8, {
          causesFire: false, // Adjust explosion properties as needed
          breaksBlocks: true // Set to false if you don't want blocks to be destroyed
        });
        dimension.spawnParticle("sndb_ww:star_explosion", hitPosition)

      }, 10); // 20 ticks = 1 second delay
    }
  });

  world.afterEvents.projectileHitBlock.subscribe((event) => {
    if (!event.projectile) return;
    if (!event.projectile.isValid()) return;

    if (event.projectile.typeId === "sndbx_ww:warden_wand_proj") {
      const hitPosition = event.projectile.location; // Get the position of the projectile
      const dimension = event.projectile.dimension; // Get the dimension of the projectile

      event.projectile.remove(); // Remove the projectile

      dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
      dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)
      dimension.spawnParticle("sndbx_ww:star_implode", hitPosition)

      // Find all entities within a 6-block radius
      const entities = dimension.getEntities({
        location: hitPosition,
        maxDistance: 20
      });

      entities.forEach(entity => {
        // Calculate the impulse vector components
        const dx = hitPosition.x - entity.location.x;
        const dz = hitPosition.z - entity.location.z;
        const dy = hitPosition.y - entity.location.y;

        // Calculate the magnitude (distance)
        const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

        // Avoid division by zero
        if (distance === 0) return;

        // Normalize direction components
        const normalizedX = dx / distance;
        const normalizedY = dy / distance;
        const normalizedZ = dz / distance;

        // Define the impulse force (adjust as needed)
        const impulseStrength = 8; // Controls overall force of the impulse
        const impulse = {
          x: normalizedX * impulseStrength,
          y: normalizedY * impulseStrength,
          z: normalizedZ * impulseStrength
        };

        // Apply the impulse to the entity
        if (entity.typeId === "minecraft:player") return;
        entity.applyImpulse(impulse);
      });
      system.runTimeout(() => {
        // Create an explosion at the hit position
        dimension.createExplosion(hitPosition, 8, {
          causesFire: false, // Adjust explosion properties as needed
          breaksBlocks: true // Set to false if you don't want blocks to be destroyed
        });
        dimension.spawnParticle("sndb_ww:star_explosion", hitPosition)
      }, 10); // 20 ticks = 1 second delay
    }
  });

  world.afterEvents.itemStartUse.subscribe((event) => {
    const itemStack = event.itemStack;
    if (itemStack.typeId !== "sndbx_ww:warden_wand") return;

    const cooldownComponent = itemStack.getComponent("cooldown");

    // Start the cooldown
    cooldownComponent?.startCooldown(event.source);

    // Get the player and its location
    const player = event.source;

    player.playAnimation("animation.sndbx_ww.player.usw_warden_wand");
    player.dimension.playSound("sndbx_ww.warden_wand.use", player.location);


    // Deduct durability
    const durabilityComponent = itemStack.getComponent("durability");
    const durabilityDeduction = 1;
    if (durabilityComponent) {
      if (durabilityComponent.damage + durabilityDeduction >= durabilityComponent.maxDurability) {
        // If durability is maxed out, play break sound and remove the item
        player.runCommand("/playsound random.break @s");
        const equipment = player.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Mainhand);
        equipment.setItem(undefined); // Clear the item from the slot
      } else {
        // Otherwise, increase damage by the deduction amount and update the item
        durabilityComponent.damage += durabilityDeduction;
        const equipment = player.getComponent('equippable')?.getEquipmentSlot(EquipmentSlot.Mainhand);
        equipment.setItem(itemStack); // Update the item with new durability
      }
    }

    player.dimension.playSound("sndbx_ww.use_wand", player.location);


    const xImpulse = 2;
    const yImpulse = 1;
    const zImpulse = 2;

    let playerViewDirection = player.getViewDirection();
    const head = player.getHeadLocation();

    let projectile = world.getDimension(player.dimension.id).spawnEntity("sndbx_ww:warden_wand_proj", offsetPositionInDirection(head, playerViewDirection, 0.5));
    projectile.clearVelocity();
    projectile.applyImpulse({ x: playerViewDirection.x * xImpulse, y: playerViewDirection.y * yImpulse, z: playerViewDirection.z * zImpulse });

  });
}

function offsetPositionInDirection(position, viewDirection, offset) {
  // Ensure the view direction vector is normalized
  const normalizedViewDirection = normalizeVector3(viewDirection);

  // Calculate the offset in each axis based on the normalized view direction
  const offsetX = normalizedViewDirection.x * offset;
  const offsetY = normalizedViewDirection.y * offset;
  const offsetZ = normalizedViewDirection.z * offset;

  // Create a new Vector3 representing the offset position
  const offsetPosition = {
    x: position.x + offsetX,
    y: position.y + offsetY,
    z: position.z + offsetZ,
  };

  return offsetPosition;
}

function normalizeVector3(vector) {
  const length = Math.sqrt(vector.x * vector.x + vector.y * vector.y + vector.z * vector.z);
  if (length !== 0) {
    return {
      x: vector.x / length,
      y: vector.y / length,
      z: vector.z / length,
    };
  } else {
    return {
      x: 0,
      y: 0,
      z: 0,
    };
  }
}