import { world, system, EquipmentSlot } from "@minecraft/server";

const weapons = [
    "sndbx_ww:warden_battle_axe",
    "sndbx_ww:warden_dagger",
    "sndbx_ww:warden_hammer",
    "sndbx_ww:warden_mace",
    "sndbx_ww:warden_shield",
    "sndbx_ww:warden_spear",
    "sndbx_ww:warden_star",
    "sndbx_ww:warden_sword"
];

export function wardenWeapons() {
    system.runInterval(() => {
        const ALL_PLAYERS = world.getAllPlayers();
        ALL_PLAYERS.forEach(player => {
            if (!player.isValid()) return;

            const equipment = player.getComponent('equippable')?.getEquipment(EquipmentSlot.Mainhand);

            if (equipment && weapons.includes(equipment.typeId)) {
                const xpOrbEntities = player.dimension.getEntities({
                    type: "minecraft:xp_orb",
                    location: player.location,
                    maxDistance: 20
                });

                xpOrbEntities.forEach(entity => {
                    // Calculate direction vector from xp_orb to player
                    const playerPosition = player.location;
                    const orbPosition = entity.location;
                    const direction = {
                        x: playerPosition.x - orbPosition.x,
                        y: playerPosition.y - orbPosition.y,
                        z: playerPosition.z - orbPosition.z
                    };

                    // Normalize the direction vector
                    const magnitude = Math.sqrt(direction.x ** 2 + direction.y ** 2 + direction.z ** 2);
                    if (magnitude === 0) return; // Avoid division by zero

                    const normalizedDirection = {
                        x: direction.x / magnitude,
                        y: direction.y / magnitude,
                        z: direction.z / magnitude
                    };

                    // Apply impulse to move the xp_orb toward the player
                    const impulseStrength = 0.1; // Adjust strength as needed
                    entity.applyImpulse({
                        x: normalizedDirection.x * impulseStrength,
                        y: normalizedDirection.y * impulseStrength,
                        z: normalizedDirection.z * impulseStrength
                    });
                });
            }
        });
    });
}
